// =====================================================
//
//   miniLA_win - Main unit
//
//   (c) miniLA Team
//
// =====================================================
//
// This is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
//
// This software is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this package; see the file COPYING.  If not, write to
// the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
// Boston, MA 02111-1307, USA.



unit dlgMain;

interface

uses Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
     StdCtrls, ComCtrls, ExtCtrls, Spin, Grids, Menus, ToolWin, Buttons, Printers,
     ImgList,
     dlgColors,
     dlgWave,
     dlgTrigger,
     dlgChannel,
     dlgHWSetup,
     uCommunication,
     uLATypes,
     uUtils,
     uDisplay,
     uFileUtils,
     uDataHold;

type
  TfrmMain = class(TForm)
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    Open1: TMenuItem;
    Save1: TMenuItem;
    SaveAs1: TMenuItem;
    N1: TMenuItem;
    Exit1: TMenuItem;
    Setup1: TMenuItem;
    Hardware1: TMenuItem;
    Channel1: TMenuItem;
    Trigger1: TMenuItem;
    StatusBar1: TStatusBar;
    SaveDialog1: TSaveDialog;
    OpenDialog1: TOpenDialog;
    Go1: TMenuItem;
    ImageList1: TImageList;
    Colors1: TMenuItem;
    OpenLayout1: TMenuItem;
    SaveLayout1: TMenuItem;
    PrintDialog1: TPrintDialog;
    PrinterSetupDialog1: TPrinterSetupDialog;
    N2: TMenuItem;
    Print1: TMenuItem;
    PrinterSetup1: TMenuItem;
    N4: TMenuItem;
    General1: TMenuItem;
    N5: TMenuItem;
    SaveLayoutDialog: TSaveDialog;
    OpenDialog2: TOpenDialog;
    Help1: TMenuItem;
    About1: TMenuItem;
    ToolBar1: TToolBar;
    btnSave: TToolButton;
    btnPrint: TToolButton;
    btnStart: TToolButton;
    btnTimeBaseInc: TToolButton;
    btnTimeBaseDec: TToolButton;
    Sep02: TToolButton;
    Panel2: TPanel;
    cbTimebase: TComboBox;
    Sep03: TToolButton;
    ExportDialog: TSaveDialog;
    ExportBINandINIFIle1: TMenuItem;
    N3: TMenuItem;
    ViewPort1: TMenuItem;
    Decoders1: TMenuItem;
    RS232Decoder: TMenuItem;
    I2CDecoder: TMenuItem;
    SPIDecoder: TMenuItem;
    btnOpen: TToolButton;
    INIExport: TMenuItem;
    VCDExport: TMenuItem;
    ImportData1: TMenuItem;
    BINImport: TMenuItem;
    PWMDecoder1: TMenuItem;
    ImportDialog: TOpenDialog;
    btnLeftCursor: TToolButton;
    btnRightCursor: TToolButton;
    btnTrigCursor: TToolButton;
    btnZoomCursor: TToolButton;
    btnZoomOut: TToolButton;
    tbZoom: TTrackBar;
    btnZoomIn: TToolButton;
    ToolButton2: TToolButton;
    ToolButton3: TToolButton;
    SetZoomPopupMenu: TPopupMenu;
    Setzoom1: TMenuItem;
    Window1: TMenuItem;
    Cascade1: TMenuItem;
    HorizontalTile1: TMenuItem;
    VerticalTile1: TMenuItem;
    ArrangeIcons1: TMenuItem;
    N6: TMenuItem;
    FrequencyMeasurement1: TMenuItem;
    btnPrevChange: TToolButton;
    btnNextChange: TToolButton;
    ToolButton5: TToolButton;
    N7: TMenuItem;
    KeyboardShortcuts1: TMenuItem;
    License1: TMenuItem;
    ools1: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure Trigger1Click(Sender: TObject);
    procedure Colors1Click(Sender: TObject);
    procedure StatusBar1DrawPanel(StatusBar: TStatusBar; Panel: TStatusPanel; const Rect: TRect);
    procedure Go1Click(Sender: TObject);
    procedure Hardware1Click(Sender: TObject);
    procedure General1Click(Sender: TObject);
    procedure SaveAs1Click(Sender: TObject);
    procedure Save1Click(Sender: TObject);
    procedure Open1Click(Sender: TObject);
    procedure OpenLayout1Click(Sender: TObject);
    procedure SaveLayout1Click(Sender: TObject);
    procedure About1Click(Sender: TObject);
    procedure ToolBtnClick(Sender: TObject);
    procedure INIExportClick(Sender: TObject);
    procedure ViewPort1Click(Sender: TObject);
    procedure RS232DecoderClick(Sender: TObject);
    procedure I2CDecoderClick(Sender: TObject);
    procedure SPIDecoderClick(Sender: TObject);
    procedure VCDExportClick(Sender: TObject);
    procedure BINImportClick(Sender: TObject);
    procedure PWMDecoder1Click(Sender: TObject);
    procedure Exit1Click(Sender: TObject);
    procedure Channel1Click(Sender: TObject);
    procedure tbZoomChange(Sender: TObject);
    procedure tbZoomUpdatePosition(pos: integer);
    procedure Cascade1Click(Sender: TObject);
    procedure HorizontalTile1Click(Sender: TObject);
    procedure VerticalTile1Click(Sender: TObject);
    procedure ArrangeIcons1Click(Sender: TObject);
    procedure Print1Click(Sender: TObject);
    procedure PrinterSetup1Click(Sender: TObject);
    procedure FrequencyMeasurement1Click(Sender: TObject);
    procedure KeyboardShortcuts1Click(Sender: TObject);
    procedure License1Click(Sender: TObject);

  private
    { Private declarations }
    FirstProgramRun: boolean;
    procedure SetTimebaseText;
  public
    { Public declarations }
    Analyzer: TAnalyzer;
    DataHold  : TDataHold;		        // data storage and access routines
    FileUtils: TFileUtils;
    destructor Destroy; override;
    procedure SaveData(Filename: string);
    procedure StateChange(Sender:TObject);
    procedure FormStart;
  end;

var
  frmMain: TfrmMain;


implementation

uses dlgSetup, dlgAbout, dlgPortStatus,
  uINIfile, dlgDecI2C, dlgDecPWM, dlgDec232, dlgDecSPI, dlgDecFreq,
  dlgListView, dlgDecDisplay;

{$R *.DFM}


//=================================================================
procedure TfrmMain.FormCreate(Sender: TObject);
//=================================================================
begin
  Caption := 'miniLA '+ProgVersion;
  OpenINIFile;					// open INI file for opening forms

  // create and initialize analyzer
  Analyzer:= TAnalyzer.Create;
  Analyzer.OnStateChange:=StateChange;

  // create and initialize data storage
  DataHold := TDataHold.Create;
  //for i:=0 to MaxDataLength-1 do
  //   DataHold.DataArray[i]:=((i and $FF) shl 16) + (i and $FFFF);
  //DataHold.InitData(MaxDataLength, true);
  DataHold.InitData(MaxDataLength, false);


  // create and initialize file routines
  FileUtils := TFileUtils.Create;


  cbTimebase.ItemIndex:= 0;
  SetTimebaseText;
end;

//=================================================================
procedure TfrmMain.FormStart;
//=================================================================
var
  windowstatemax : boolean;
  wwidth : integer;
  wheight : integer;
begin
  frmChannel.btnOKClick(nil);
  frmMain.Analyzer.Init;
  frmList.Initialize;
  FirstProgramRun := INIFile.ReadBool('MAIN', 'FIRSTPROGRAMRUN', true);

  windowstatemax := INIFile.ReadBool('MAIN', 'WINDOWSTATEMAX', false);
  wheight := INIFile.ReadInteger('MAIN', 'HEIGHT', 600);
  wwidth := INIFile.ReadInteger('MAIN', 'WIDTH', 900);

  frmMain.Height := wheight;
  frmMain.Width := wwidth;


{  if windowstatemax then
   begin
     frmMain.WindowState := wsMaximized;
    //ShowWindow(Handle, SW_MAXIMIZE);
   end;
}

  if FirstProgramRun then
   begin
     About1Click(self);
     FirstProgramRun := false;
   end;
end;

//=================================================================
destructor TfrmMain.Destroy;
//=================================================================
begin
  INIFile.WriteBool('MAIN', 'FIRSTPROGRAMRUN', FirstProgramRun);
  INIFile.WriteInteger('MAIN', 'HEIGHT', frmMain.Height);
  INIFile.WriteInteger('MAIN', 'WIDTH', frmMain.Width);
  if frmMain.WindowState = wsMaximized then
     INIFile.WriteBool('MAIN', 'FWINDOWSTATEMAX', true)
  else
     INIFile.WriteBool('MAIN', 'FWINDOWSTATEMAX', false);

  Analyzer.Free;
  inherited Destroy;
  CloseINIFile;
end;

//=================================================================
procedure TfrmMain.FormResize(Sender: TObject);
//=================================================================
begin
   if (csDestroying in ComponentState) then exit;

end;

//=================================================================
procedure TfrmMain.Exit1Click(Sender: TObject);
//=================================================================
begin
   Close;
end;

//=================================================================
procedure TfrmMain.Colors1Click(Sender: TObject);
//=================================================================
begin
  if frmColors.ShowModal=mrOk then
   begin
   end;
end;



//=================================================================
procedure TfrmMain.tbZoomChange(Sender: TObject);
//=================================================================
begin
   frmWave.LogPanel.SetZoom(IntExp2(tbZoom.Position)*100);
end;

// updates tbZoom without firing the OnChange event
//=================================================================
procedure TfrmMain.tbZoomUpdatePosition(pos: integer);
//=================================================================
var
   oldNotify	:TNotifyEvent;
begin
   oldNotify := tbZoom.OnChange;
   tbZoom.OnChange := nil;
   tbZoom.Position:=pos;
   tbZoom.OnChange := oldNotify;
end;


//=================================================================
procedure TfrmMain.ViewPort1Click(Sender: TObject);
//=================================================================
begin
   if not(frmPortStatus.Visible) then
      frmPortStatus.Show
   else
      frmPortStatus.Hide;
end;

//=================================================================
procedure TfrmMain.SetTimebaseText;
//=================================================================
var
   i		: byte;
   per		: real;
   akt_item	: byte;
begin
   akt_item := cbTimebase.ItemIndex;
   cbTimebase.Clear;					// clear all list items
   if not(Analyzer.StateAnalysis) then
    begin
      for i:=0 to 18 do
       begin
          if Analyzer.Timebase_freq[i] <> 0 then
             per := 1/Analyzer.Timebase_freq[i]
          else
             per := 0;
          cbTimebase.Items.Add(TmToStr(per,2)+'s')
       end;
      cbTimebase.Items.Add('EXT. CLK');
      cbTimebase.Enabled := true;
      btnTimebaseDec.Enabled := true;
      btnTimebaseInc.Enabled := true;
      cbTimebase.ItemIndex := akt_item;
    end
   else
    begin
      if Analyzer.Frequency <> 0 then
         per := 1/Analyzer.Frequency
      else
         per := 0;
      cbTimebase.Items.Add(TmToStr(per,2)+'s');
      cbTimebase.ItemIndex := 0;
      cbTimebase.Enabled := false;
      btnTimebaseDec.Enabled := false;
      btnTimebaseInc.Enabled := false;
    end;
end;

//=================================================================
procedure TfrmMain.Trigger1Click(Sender: TObject);
//=================================================================
begin
   if frmTrigger.ShowModal=mrOk then
    begin
      PrepareDispData(frmWave.LogPanel.Display.DataSet);	// update signal names (might change in trigger menu)
      frmList.Initialize;
    end;
end;


//=================================================================
procedure TfrmMain.Hardware1Click(Sender: TObject);
//=================================================================
begin
   Analyzer.Timer.Enabled := false;			// no IO access during HW setup
   if frmHWSetup.ShowModal = mrOK then
      SetTimebaseText;					// update timebase values (frequency might be changed)
   Analyzer.Timer.Enabled := true;			// re-enable timer
end;

//=================================================================
procedure TfrmMain.General1Click(Sender: TObject);
//=================================================================
begin
  if frmSetup.ShowModal=mrOk then begin
  end;
end;


//=================================================================
procedure TfrmMain.Channel1Click(Sender: TObject);
//=================================================================
begin
  if frmChannel.ShowModal=mrOk then
   begin
     frmWave.LogPanel.Initialize(frmWave.LogPanel.Zoom, frmWave.LogPanel.Position, frmWave.LogPanel.Cursors[2].Position);
     frmList.Initialize;
   end;
end;


//=================================================================
procedure TfrmMain.PrinterSetup1Click(Sender: TObject);
//=================================================================
begin
  PrinterSetupDialog1.Execute;
end;


//=================================================================
procedure TfrmMain.Print1Click(Sender: TObject);
//=================================================================
var BC,SC       :TColor;
    sigwidth, width    : integer;
begin
  if PrintDialog1.Execute then begin
    Printer.BeginDoc;

    // draw into printer's canvas
    BC:=frmWave.LogPanel.Display.BackColor;
    SC:=frmWave.LogPanel.Display.ScaleColor;
    frmWave.LogPanel.Display.BackColor:=clWhite;
    frmWave.LogPanel.Display.ScaleColor:=clBlack;
    if Printer.PageWidth > Printer.PageHeight then
     begin
       // landscape

       width := 18*Printer.PageWidth div 20;
       sigwidth :=  width * frmWave.sgSigNames.Width div frmWave.Width;

       frmWave.LogPanel.Display.DrawNames
       (
          Printer.PageWidth div 20,
          Printer.PageHeight div 20,
          sigwidth,
          17*Printer.PageHeight div 20,
          Printer.Canvas
       );

       frmWave.LogPanel.Display.Redraw
       (
          sigwidth+Printer.PageWidth div 20,
          Printer.PageHeight div 20,
          width-sigwidth,
          17*Printer.PageHeight div 20,
          Printer.Canvas
       );
     end

    else
     begin
       // Portrait

       width := 12*Printer.PageWidth div 14;
       sigwidth :=  width * frmWave.sgSigNames.Width div frmWave.Width;

       frmWave.LogPanel.Display.DrawNames
       (
          Printer.PageWidth div 20,
          Printer.PageHeight div 20,
          sigwidth,
          9*Printer.PageHeight div 20,
          Printer.Canvas
       );

       frmWave.LogPanel.Display.Redraw
       (
          sigwidth+Printer.PageWidth div 14,
          Printer.PageWidth div 14,
          width-sigwidth,
          9*Printer.PageWidth div 14,
          Printer.Canvas
       );

     end;

    frmWave.LogPanel.Display.BackColor:=BC;
    frmWave.LogPanel.Display.ScaleColor:=SC;

    // redraw on screen to restore internal variables (ZoomRatio etc.)
    frmWave.LogPanel.Display.Redraw(0,0,frmWave.LogPanel.Display.Width,frmWave.LogPanel.Display.Height,frmWave.LogPanel.Display.Bitmap.Canvas);

    // debug info
    //StatusBar1.Panels[1].Text:=IntToStr(Printer.PageWidth);
    //StatusBar1.Panels[2].Text:=IntToStr(Printer.PageHeight);
    Printer.EndDoc;
  end;
end;



//=================================================================
procedure TfrmMain.Save1Click(Sender: TObject);
//=================================================================
begin
  if FileUtils.FileName='' then
    SaveAs1Click(Sender)
  else
    SaveData(FileUtils.Filename);
end;

//=================================================================
procedure TfrmMain.SaveAs1Click(Sender: TObject);
//=================================================================
begin
  SaveDialog1.FileName:=FileUtils.FileName;
  if SaveDialog1.Execute then
    SaveData(SaveDialog1.FileName);
end;


//=================================================================
procedure TfrmMain.SaveData(Filename: string);
//=================================================================
var s:string;
begin
   frmMain.StatusBar1.Panels[5].Text := 'Saving data ...';
//   frmMain.StatusBar1.Repaint;
   frmMain.StatusBar1.Update;

   if FileUtils.WriteDataToFile(Filename, false)<0 then
     Application.MessageBox('A file save error occured. Please try again.',
       'File Save Error',mb_IconError or mb_Ok)
   else
    begin
      frmMain.Caption:='miniLA '+ProgVersion+' - '+FileUtils.FileName;
      if frmSetup.SaveLayoutWFile then
       begin
         s:=Copy(FileUtils.FileName,1,
	   Length(FileUtils.FileName)-Length(ExtractFileExt(FileUtils.FileName)));
         frmChannel.SaveLayout(s+'.aly');
       end;
    end;

   frmMain.StatusBar1.Panels[5].Text := '  ';
//   frmMain.StatusBar1.Repaint;
   frmMain.StatusBar1.Update;
end;


//=================================================================
procedure TfrmMain.Open1Click(Sender: TObject);
//=================================================================
var s,s2: string;
begin
  if OpenDialog1.Execute then
   begin
    frmMain.StatusBar1.Panels[5].Text := 'Loading data ...';
    frmMain.StatusBar1.Repaint;

    if FileUtils.ReadDataFromFile(OpenDialog1.FileName)<0 then
      Application.MessageBox('An error occured while loading data.','Open File Error',
	mb_IconError or mb_Ok)
    else begin
      s:=Copy(OpenDialog1.FileName,1, Length(OpenDialog1.FileName)-Length(ExtractFileExt(OpenDialog1.FileName)));
      if frmSetup.LoadLayoutWFile and FileExists(s+'.aly') then
	if frmSetup.ConfirmBefore then begin
	  s2:='Do you want to open the layout file "'+s+'.aly" too?';
	  if Application.MessageBox(PChar(s2),'Auto-open layout',mb_IconQuestion or mb_YesNo)=mrYes
	    then if frmChannel.LoadLayout(s+'.aly')<0 then
	      Application.MessageBox('An error occured while loading layout file.','Open layout',
		mb_IconError);
	end else
	  if frmChannel.LoadLayout(s+'.aly')<0 then
	    Application.MessageBox('An error occured while loading layout file.','Open layout',
	      mb_IconError);

      frmMain.Caption:='miniLA '+ProgVersion+' - '+FileUtils.FileName;

      frmWave.LogPanel.Initialize(100, frmWave.Logpanel.Cursors[2].Position, frmWave.Logpanel.Cursors[2].Position);
      frmList.Initialize;

    end;

    frmMain.StatusBar1.Panels[5].Text := '';
    frmMain.StatusBar1.Repaint;
   end;

end;

//=================================================================
procedure TfrmMain.OpenLayout1Click(Sender: TObject);
//=================================================================
begin
  if OpenDialog2.Execute then
    if frmChannel.LoadLayout(OpenDialog2.FileName)<0 then
      Application.MessageBox('An error occured while loading layout.','Open File Error',
	mb_IconError or mb_Ok)
    else begin
      frmWave.LogPanel.Initialize(frmWave.LogPanel.Zoom, frmWave.LogPanel.Position, frmWave.LogPanel.Cursors[2].Position);
      frmList.Initialize;
    end;
end;


//=================================================================
procedure TfrmMain.SaveLayout1Click(Sender: TObject);
//=================================================================
begin
  if SaveLayoutDialog.Execute then
    if frmChannel.SaveLayout(SaveLayoutDialog.FileName)<0 then
      Application.MessageBox('A file save error occured. Please try again.',
	'File Save Error',mb_IconError or mb_Ok);
end;

//=================================================================
procedure TfrmMain.BINImportClick(Sender: TObject);
//=================================================================
begin
  if ImportDialog.Execute then
   begin
    frmMain.StatusBar1.Panels[5].Text := 'Importing BIN file ...';
    frmMain.StatusBar1.Repaint;

    if FileUtils.ReadBINFile(ImportDialog.FileName)<0 then
      Application.MessageBox('An error occured while loading data.','Open File Error',
	mb_IconError or mb_Ok)
    else
     begin
       with frmWave.LogPanel.Display do
        begin
          Redraw(0,0,Width,Height,Bitmap.Canvas);
          Repaint;
	end;
     end;

    frmWave.LogPanel.UpdateCursors;

    frmMain.StatusBar1.Panels[5].Text := '';
    frmMain.StatusBar1.Repaint;
   end;

end;


//=================================================================
procedure TfrmMain.INIExportClick(Sender: TObject);
//=================================================================
var s: string;
begin
  s:=Copy(FileUtils.FileName,1, Length(FileUtils.FileName)-Length(ExtractFileExt(FileUtils.FileName)));
  ExportDialog.Filename := s;
  ExportDialog.Filter := 'Logic Analyzer Data File (*.bin)|*.bin|All files (*.*)|*.*';
  if ExportDialog.Execute then
   begin
     frmMain.StatusBar1.Panels[5].Text := 'Exporting BIN file ...';
     frmMain.StatusBar1.Repaint;

     if FileUtils.WriteDataToFile(ExportDialog.Filename, true)<0 then
	Application.MessageBox('A file save error occured. Please try again.',
	  'File Save Error',mb_IconError or mb_Ok)
     else
      begin
	s:=Copy(ExportDialog.FileName,1, Length(ExportDialog.FileName)-Length(ExtractFileExt(ExportDialog.FileName)));
	ExportDialog.Filename := s;
	ExportDialog.Filter := 'Logic Analyzer INI File (*.ini)|*.ini|All files (*.*)|*.*';
	if ExportDialog.Execute then
	 begin
	   frmMain.StatusBar1.Panels[5].Text := 'Exporting INI file ...';
	   frmMain.StatusBar1.Repaint;

	   if FileUtils.WriteINIFile(ExportDialog.Filename)<0 then
	      Application.MessageBox('A file save error occured. Please try again.',
		'File Save Error',mb_IconError or mb_Ok)
	 end;
       end;

     frmMain.StatusBar1.Panels[5].Text := '';
     frmMain.StatusBar1.Repaint;
   end;

end;

//=================================================================
procedure TfrmMain.VCDExportClick(Sender: TObject);
//=================================================================
var s: string;
begin
  s:=Copy(FileUtils.FileName,1, Length(FileUtils.FileName)-Length(ExtractFileExt(FileUtils.FileName)));
  ExportDialog.Filename := s;
  ExportDialog.Filter := 'VCD File (*.vcd)|*.vcd|All files (*.*)|*.*';
  if ExportDialog.Execute then
   begin
     frmMain.StatusBar1.Panels[5].Text := 'Exporting VCD file ...';
     frmMain.StatusBar1.Repaint;

     if FileUtils.WriteVCDFile(ExportDialog.Filename)<0 then
	Application.MessageBox('A file save error occured. Please try again.',
	  'File Save Error',mb_IconError or mb_Ok);

     frmMain.StatusBar1.Panels[5].Text := '';
     frmMain.StatusBar1.Repaint;
   end;

end;

//=================================================================
procedure TfrmMain.I2CDecoderClick(Sender: TObject);
//=================================================================
begin
   frmDecI2C.ShowModal;
end;

//=================================================================
procedure TfrmMain.PWMDecoder1Click(Sender: TObject);
//=================================================================
begin
   frmDecPWM.ShowModal;
end;

//=================================================================
procedure TfrmMain.RS232DecoderClick(Sender: TObject);
//=================================================================
begin
   frmDec232.ShowModal;
end;

//=================================================================
procedure TfrmMain.SPIDecoderClick(Sender: TObject);
//=================================================================
begin
   frmDecSPI.ShowModal;
end;


//=================================================================
procedure TfrmMain.FrequencyMeasurement1Click(Sender: TObject);
//=================================================================
begin
   frmDecFreq.ShowModal;
end;


//=================================================================
procedure TfrmMain.Cascade1Click(Sender: TObject);
//=================================================================
begin
   Cascade;
end;

//=================================================================
procedure TfrmMain.HorizontalTile1Click(Sender: TObject);
//=================================================================
begin
   TileMode := tbHorizontal;
   Tile;
end;

//=================================================================
procedure TfrmMain.VerticalTile1Click(Sender: TObject);
//=================================================================
begin
   TileMode := tbVertical;
   Tile;
end;

//=================================================================
procedure TfrmMain.ArrangeIcons1Click(Sender: TObject);
//=================================================================
begin
   ArrangeIcons;
end;


//=================================================================
procedure TfrmMain.KeyboardShortcuts1Click(Sender: TObject);
//=================================================================
begin
   Application.HelpJump('KeyboardShortcuts');
end;

//=================================================================
procedure TfrmMain.License1Click(Sender: TObject);
//=================================================================
begin
     Application.HelpJump('GNUGPL');
end;


//=================================================================
procedure TfrmMain.About1Click(Sender: TObject);
//=================================================================
begin
  AboutBox.ShowModal;
end;

//******************************************************************************
//******************************************************************************
//******************************************************************************

//=============================================
procedure TfrmMain.ToolBtnClick(Sender: TObject);
//=============================================
begin
  case (Sender as TToolButton).Tag of
    1: Open1Click (Self);
    2: Save1Click (Self);
    3: Print1Click(Self);
    4: if cbTimebase.ItemIndex> 0 then begin
	 cbTimebase.ItemIndex:= cbTimebase.ItemIndex-1;
       end;
    5: if cbTimebase.ItemIndex<26 then begin
	 cbTimebase.ItemIndex:= cbTimebase.ItemIndex+1;
       end;
    6: Go1Click(Self);
    7: frmWave.ViewCur1Click(Self);
    8: frmWave.ViewCur2Click(Self);
    9: frmWave.ViewTriggerClick(Self);
   10: frmWave.ZoomCursorsClick(Self);
   11: frmWave.ZoomInClick(Self);
   12: frmWave.ZoomOutClick(Self);
   13: frmWave.Centertopreviouschange1Click(self);
   14: frmWave.Centertonextchange1Click(self);
  end;
end;

//******************************************************************************
//******************************************************************************
//******************************************************************************

//=================================================================
procedure TfrmMain.Go1Click(Sender: TObject);
//=================================================================
begin
  if Analyzer.State=tsReady then
   begin
    btnOpen.Enabled := False;
    btnSave.Enabled := False;
    btnTimeBaseInc.Enabled := False;
    btnTimeBaseDec.Enabled := False;
    File1.Enabled := False;
    Setup1.Enabled := False;
    Trigger1.Enabled := False;
    Help1.Enabled := False;
    ViewPort1.Enabled := False;
    Decoders1.Enabled := False;
    cbTimebase.Enabled := False;
    //frmDecDisplay.Enabled := False;

    btnStart.ImageIndex := 15;
    Go1.Caption := 'Stop!';

    // set all parameters of the analyzer
    Analyzer.Timebase_sel := cbTimebase.ItemIndex;
    Analyzer.TriggerDelay := frmTrigger.TrigDelay;
    Analyzer.TriggerLength:= frmTrigger.TrigLen;
    Analyzer.TriggerMask  := frmTrigger.TrigMask;
    Analyzer.TriggerValue := frmTrigger.TrigValue;
    Analyzer.TriggerEdge  := frmTrigger.TrigEdge;
    Analyzer.TriggerSize  := frmTrigger.TrigSize;
    Analyzer.InvIntTrig   := frmTrigger.InvIntTrig;
    Analyzer.ExtTrig_en   := frmTrigger.ExtTrig_en;
    Analyzer.ExtTrig_val  := frmTrigger.ExtTrig_val;
    Analyzer.Start;
   end
  else
   begin
    Analyzer.Stop;
   end;
end;

//=================================================================
procedure TfrmMain.StateChange;
//=================================================================
var i:integer;
    s_text:string;
begin
   case Analyzer.State of
      tsNotDetected:
       begin
	 StatusBar1.Panels[0].Text:='HW Not Detected';

	 // enable all control
	 btnOpen.Enabled := True;
	 btnSave.Enabled := True;
	 btnPrint.Enabled := True;
	 btnTimeBaseInc.Enabled := True;
	 btnTimeBaseDec.Enabled := True;
	 File1.Enabled := True;
	 Setup1.Enabled  := True;
	 Trigger1.Enabled := True;
	 cbTimebase.Enabled := True;
	 Help1.Enabled := True;
         ViewPort1.Enabled := True;
	 Decoders1.Enabled := True;
	 //frmDecDisplay.Enabled := True;

	 btnStart.ImageIndex :=14;
	 Go1.Enabled := false;
	 btnStart.Enabled := false;
       end;
      tsReady:
       begin
         if not Analyzer.StateAnalysis then
            s_text := 'TA'
         else
            s_text := 'SA';
	 StatusBar1.Panels[0].Text:= Format('FW %d.%d(%s) Ready ...',[(Analyzer.Version and $70) shr 4,Analyzer.Version and $F,s_text]);

	 if (Analyzer.Old_State = tsNotDetected) or (Analyzer.Old_State = tsReady) then
	  begin
	    Go1.Enabled := true;
	    btnStart.Enabled := true;
            SetTimebaseText;			// update timebase text (time/state analysis)
            if not Analyzer.StateAnalysis then
               frmChannel.SetEnabledChannels(32, false)
            else
	       frmChannel.SetEnabledChannels(16, false);
	  end
	 else
	  begin
	    // sampling complete, enable all control
	    btnOpen.Enabled := True;
	    btnSave.Enabled := True;
	    btnPrint.Enabled := True;
	    btnTimeBaseInc.Enabled := True;
	    btnTimeBaseDec.Enabled := True;
	    File1.Enabled := True;
	    Setup1.Enabled  := True;
	    Trigger1.Enabled := True;
	    cbTimebase.Enabled := True;
	    Help1.Enabled := True;
            ViewPort1.Enabled := True;
	    Decoders1.Enabled := True;
	    //frmDecDisplay.Enabled := True;

	    btnStart.ImageIndex :=14;
	    Go1.Caption := 'Go!';

	    // get sampled data
	    for i:=0 to Analyzer.DataLength-1 do
	       frmMain.DataHold.DataArray[i]:=Analyzer.DataArray[i];

	    frmWave.LogPanel.Timebase := Analyzer.Timebase_freq[Analyzer.Timebase_sel];
            frmMain.DataHold.InitData(Analyzer.Datalength, Analyzer.StateAnalysis);
            frmWave.LogPanel.Initialize(frmWave.LogPanel.Zoom, frmWave.LogPanel.Position, frmMain.DataHold.GetPosition(Analyzer.TriggerPos));
            frmList.Initialize;

	    // clear results of decoder (obsolete, new data present)
	    frmDecDisplay.lbResults.Clear;
	  end;
       end;

     tsWaitForTrigger:
	StatusBar1.Panels[0].Text:='Capturing...';

     tsPosttrigger:
	StatusBar1.Panels[0].Text:='Capturing...';

     tsReading:
	StatusBar1.Panels[0].Text:='Reading...';
   end;
  StatusBar1.Refresh;
end;


//=================================================================
// Status bar
//=================================================================



//=================================================================
procedure TfrmMain.StatusBar1DrawPanel(StatusBar: TStatusBar; Panel: TStatusPanel; const Rect: TRect);
//=================================================================
var i:byte;
begin
  i:=0;
  while i<5 do begin
    if Panel=StatusBar.Panels[i] then break;
    inc(i);
  end;
  with StatusBar.Canvas do begin
    Brush.Color:=clWindow;
    Rectangle(Rect.Left,Rect.Top,Rect.Right,Rect.Bottom);
    case i of
      0: Font.Color:=clWindowText;
      1: Font.Color:=frmWave.LogPanel.Cursors[2].ColorC;
      2: begin
           Font.Color:=frmWave.LogPanel.Cursors[0].ColorC;
           if (frmWave.LogPanel.ActiveCursor = 0) then
              Font.Style := [fsBold]
           else
              Font.Style := []
         end;
      3: begin
           Font.Color:=frmWave.LogPanel.Cursors[1].ColorC;
           if frmWave.LogPanel.ActiveCursor = 1 then
              Font.Style := [fsBold]
           else
              Font.Style := []
         end;
    else Font.Color:=clWindowText;
    end;
    TextRect(Rect,Rect.Left,Rect.Top,Panel.Text);
  end;
end;








end.
