// =====================================================
//
//   miniLA_win - Display unit
//
//   (c) miniLA Team
//
// =====================================================
//
// This is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
//
// This software is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this package; see the file COPYING.  If not, write to
// the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
// Boston, MA 02111-1307, USA.




unit uDisplay;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  stdctrls,extctrls, WOwnrScrllBar,
  uLATypes,
  uUtils;

procedure PrepareDispData(var DispData:TDisplayData);
function ChannelPosValueToStr(pos: int64; channel: integer):string;
function ChannelValueToStr(value: cardinal; channel: integer):string;

type

  // cursors
  TCursor=class(TGraphicControl)
  protected
    procedure Paint; override;
  public
    Horizontal : boolean;		// horizontal/vertical cursor
    Enabled    : boolean;
    Moving     : boolean;
    Zooming    : boolean;
    Position   : int64;
    Number     : byte;
    ColorC     : TColor;
    constructor Create(AOwner:TComponent; No:byte); reintroduce;
    procedure   MouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer); reintroduce;
    procedure   MouseUp  (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer); reintroduce; dynamic;
    procedure   MouseDn  (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer); dynamic;
  end;



  // waveforms
  TLogDisplay=class(TGraphicControl)
  private
    FOnMouseMoved  : TNotifyEvent;

  public
    { Public declarations }
    Bitmap    : TBitmap;
    DataSet   : TDisplayData; 		// konfigurace zobrazeni
    SignalHeight: integer;		// height of signal
    BackColor: TColor;
    GroupColor: TColor;
    ScaleColor: TColor;
    GridColor: TColor;
    constructor Create(AOwner:TComponent); override;
    destructor Destroy; override;
    procedure  SetColors (Back,Group,Scale,Grid:TColor);
    procedure  Paint; override;
    procedure  Redraw(SX,SY,W,H:integer;Canvas:TCanvas); virtual;
    procedure  DrawNames(SX,SY,W,H:integer;Canvas:TCanvas);
    procedure  ShowScale (SX,SY,W,H:integer;Canvas:TCanvas;Fill:boolean);
    procedure  MouseMove (Shift: TShiftState; X, Y: Integer); override;
  published
    property   OnMouseMove : TNotifyEvent read FOnMouseMoved  write FOnMouseMoved  default nil;	// redeclaration to TNotifyEvent
  end;



  // LogPanel
  TLogPanel=class(TCustomControl)
  private
    procedure   WMGetDlgCode    (var Msg: TWMGetDlgCode); message WM_GETDLGCODE;
    procedure   KeyDown         (var Key: Word; Shift: TShiftState); override;
    procedure   MouseWheel      (var Msg: TWMMouseWheel); message CM_MouseWheel;
    procedure   fSetZoomPos     (Zoom_: real; Pos_:int64);
  protected
  public
    { Public declarations }
    FZoom     : real; 			// zoom (range 100 to MaxZoom [%])
    FMaxZoom  : real;
    FPosition : int64; 			// pocatecni pozice dat ve vzorcich
    ZoomRatio : single;			// number of data per pixel for given zoom (FZoom)

    TimeBase    : double;
    Display     : TLogDisplay;
    Cursors     : array[0..4] of TCursor;
    StatusText  : TLabel;
    SelDataset  : integer;	        // signal number of signal under cursor focus
    ActiveCursor: integer;              // selected (active) cursor 0 or 1
    ActiveDataset: integer;             // selected (active) dataset

    constructor Create(AOwner:TComponent); reintroduce;
    destructor  Destroy; override;
    procedure   Paint; reintroduce; virtual;
    procedure   SetFocus; override;
    procedure   OnResize;

    procedure   Initialize       (Zoom_: real; Pos_: int64; tpos: int64);
    procedure   UpdateCursors;
    procedure   SetActiveDataset (No : integer);
    procedure   SetActiveCursor  (No : integer);
    procedure   SetZoom          (Value: real);
    procedure   SetPosition      (Value: int64);
    procedure   SetCenterPosition(Value: int64);
    procedure   SetViewRange     (Pos1, Pos2: int64);
    procedure   SetZoomPos       (Zoom_: real; Pos_:int64);
    procedure   SetCursorToHalf  (No   : byte);
    procedure   ShowCursor       (No   : byte);
    procedure   SetCursorPosition(No   : byte; Pos:int64);
    procedure   SetSnapCursorPosition(No   : byte; Pos:int64);
    procedure   CenterChange	 (go_forward: boolean);

    function    PixelToAddr(Position:int64; Point:integer):int64;
    function    CPixelToAddr(Position:int64; Point:integer):int64;
    function    AddrToPixel(Position:int64; Address:int64):integer;

    property    Zoom    : real read FZoom     write SetZoom;
    property    MaxZoom : real read FMaxZoom;
    property    Position: int64 read FPosition write SetPosition default 0;
  end;

implementation
uses dlgMain, dlgWave, dlgSetup, dlgChannel, dlgListView, uINIFile;



//==============================================================================
//
//  TCursor
//
//==============================================================================


//=================================================================
constructor TCursor.Create(AOwner:TComponent; No:byte);
//=================================================================
begin
  inherited Create(AOwner);
  Horizontal := false;
  Enabled    := true;
  Moving     := false;
  Zooming    := false;
  Number     := No;
  Position   := int64(0);

  OnMouseMove:= MouseMove;
  OnMouseUp  := MouseUp;
  OnMouseDown:= MouseDn;

  if Number<2 then
    Cursor:=crSizeWE;	// cursor <->
end;

//=================================================================
// Mouse moved above the cursor
// (moving cursor when mouse key pressed)
//=================================================================
procedure TCursor.MouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
var Pos: integer;
    Own: TLogPanel;
begin
  Own:= (Owner as TLogPanel);

  if Moving then					// changing position of cursor
   begin
     if Horizontal then
      begin  // horizontal cursor
        Pos:=Y+Top-(Height+1) div 2;
        if Pos<(Height+1) div 2 then Pos:=(Height+1) div 2;
        if Pos>Parent.Height-(Height+1) div 2-16 then Pos:=Parent.Height-(Height+1) div 2-16;
        Top:=Pos;
      end
     else // vertical cursor
      begin
        Pos:=X+Left-(Width+1) div 2-2;
        if Pos<0 then Pos:=0;
        if Pos>Parent.Width-Width-5 then Pos:=Parent.Width-Width-5;
	Own.SetCursorPosition(Number,Own.CPixelToAddr(Own.Position,Pos));
      end;
   end;

  Own.Display.MouseMove(Shift,X+Left,Y+Top);		// propagate to display
end;

//=================================================================
// if above cursor and left button - start moving cursor
// right button - open popup menu
// middle button - zoom in
//=================================================================
procedure TCursor.MouseDn(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var P:TPoint;
    No : integer;
begin
   if (Button=mbLeft) then
    begin
      if (Number<2) and not (Owner as TLogPanel).Cursors[3].Zooming then
       begin
         Moving:=true;
         (Owner as TLogPanel).Cursors[3].Visible:=false;
         (Owner as TLogPanel).Cursors[4].Visible:=false;
       end
{      else
       begin
         // set active dataset
         No:=(Y+Top-3) div (Owner as TLogPanel).Display.SignalHeight;
         (Owner as TLogPanel).SetActiveDataset(No);
       end;}
    end
   else if (Button=mbMiddle) and not Moving then
    begin
      (Owner as TLogPanel).Cursors[3].Zooming:=true;
      with (Owner as TLogPanel).Cursors[4] do
       begin
	 horizontal:= false;
	 Top       := (Owner as TLogPanel).Cursors[3].Top;
	 Left      := (Owner as TLogPanel).Cursors[3].Left;
	 Width     := (Owner as TLogPanel).Cursors[3].Width;
	 Height    := (Owner as TLogPanel).Cursors[3].Height;
	 Position  := (Owner as TLogPanel).Cursors[3].Position;
       end;
      MouseCapture := true;
      (Owner as TLogPanel).StatusText.Caption:= 'Select zoom area';
    end
   else if (Button=mbRight) then
    begin
      P.x:=X;
      P.y:=Y;
      P:=ClientToScreen(P);
      frmWave.DisplayPopupMenu.Popup(P.X,P.Y);
    end;

  (Owner as TLogPanel).SetFocus;
end;

//=================================================================
// end of possible cursor move, enable mouse cursors
//=================================================================
procedure TCursor.MouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if (Owner as TLogPanel).Cursors[3].Zooming then
   begin
      MouseCapture := false;
      with (Owner as TLogPanel).Cursors[4] do
       begin
	 horizontal:= true;
	 Top       := Y;
	 Left      := 2;
	 Height    := 1;
	 width	   := (Parent as TLogPanel).width;
       end;
      (Owner as TLogPanel).Cursors[3].Zooming := false;
      (Owner as TLogPanel).SetViewRange((Owner as TLogPanel).Cursors[3].Position,
					(Owner as TLogPanel).Cursors[4].Position);
   end;
  Moving:=false;
  (Owner as TLogPanel).Cursors[3].Visible:=true;
  (Owner as TLogPanel).Cursors[4].Visible:=true;

  // snap cursor to edges
  if Number<2 then
     (Owner as TLogPanel).SetSnapCursorPosition(Number,Position);
end;

//=================================================================
// Draw cursor
//=================================================================
procedure TCursor.Paint;
var R:TRect;
begin
  with Canvas do
   begin
    if Horizontal then
     begin
      Pen.Color:=ColorC;
      MoveTo(0,(Height+1) div 2-1);
      LineTo(Width,(Height+1) div 2-1);
     end
    else
     begin
      Pen.Color  :=ColorC;
      Brush.Color:= ColorC;
      MoveTo(((Width+1) div 2)-1,0);
      LineTo(((Width+1) div 2)-1,Height);
      if (Owner as TLogPanel).ActiveCursor = Number then
       begin
         MoveTo(((Width+1) div 2),0);
         LineTo(((Width+1) div 2),Height);
       end;
     end;
   end;
end;



//==============================================================================
//
//  TLogDisplay
//
//==============================================================================

//=================================================================
constructor TLogDisplay.Create;
//=================================================================
var i:integer;
begin
  inherited Create(AOwner);
  SignalHeight:=1;

  for i:=0 to MaxChannels-1 do
     Dataset.Data[i].DispVal:=0;

  Bitmap:=TBitmap.Create;
  BackColor:=INIFile.ReadInteger('GUI','BACKGROUND_COLOR', clBlack);
  GroupColor:=INIFile.ReadInteger('GUI','GROUP_COLOR', (51*256+185)*256+85);
  ScaleColor:=INIFile.ReadInteger('GUI','SCALE_COLOR', clWhite);
  GridColor:=INIFile.ReadInteger('GUI','GRID_COLOR', $00444444);
end;

//=================================================================
destructor TLogDisplay.Destroy;
//=================================================================
begin
  INIFile.WriteInteger('GUI','BACKGROUND_COLOR', BackColor);
  INIFile.WriteInteger('GUI','GROUP_COLOR', GroupColor);
  INIFile.WriteInteger('GUI','SCALE_COLOR', ScaleColor);
  INIFile.WriteInteger('GUI','GRID_COLOR', GridColor);
  Bitmap.Free;
  inherited Destroy;
end;

//=================================================================
// Mouse moved above the waveform window
// - update position of cursors
// - update signal values
// - update caption
//=================================================================
procedure TLogDisplay.MouseMove(Shift: TShiftState; X, Y: Integer);
var s:string;
    i:int64;
    j,No:integer;
    m:cardinal;
    Own:TLogPanel;
begin
  Own:= (Owner as TLogPanel);

  // cursor coordinate
  i:=X-5;
  if i>Width-10 then i:=Width-10;
  if i<0 then i:=0;

  // cursor position
  i:=Own.CPixelToAddr(Own.Position,i);
  if i<0 then i:=0;
  if i>frmMain.DataHold.MaxPosition then i:=frmMain.DataHold.MaxPosition;

  // move vertical cursor
  Own.SetCursorPosition(3,i);

  // move horizontal cursor and print status line text
  if not(Own.Cursors[3].Zooming) then
   begin
     if Y<2 then Y:=2;
     if Y>Height-3 then Y:=Height-3;
     Own.Cursors[4].Top:=Y;

     if DataSet.NumberOfLines=0 then begin
       Own.StatusText.Caption:='All signals are disabled';
       Own.SelDataset := -1;
       exit;
     end;

     s:='Main: '+IntToDecStr(i-Own.Cursors[2].Position)+' ('+TmToStr((i-Own.Cursors[2].Position)/Own.TimeBase,2)+'s)';

     // show signal(+weight)/group value in status text
     No:=(Y-3) div SignalHeight;
     if No<0 then No:=0;
     if No<DataSet.NumberOfLines then
      begin
        Own.SelDataset := No;
        if DataSet.Data[No].Group then
	   s:=s+' - Group: '+DataSet.Data[No].Name+' <'+ChannelPosValueToStr(i, No)+'>'
	else
         begin
           j:=0;
           m:=DataSet.Data[No].Mask;
           while m<>0 do begin
             m:=m shr 1;
             inc(j);
           end;
           s:=s+' - Signal: '+IntToDecStr(j)+'. '+DataSet.Data[No].Name;
	   if frmMain.DataHold.GetSampleValue(i) and DataSet.Data[No].Mask<>0 then s:=s+' <log. 1>'
                                                   else s:=s+' <log. 0>';
         end;
      end
     else
       Own.SelDataset := -1;

     // print caption
     Own.StatusText.Caption:= s;
   end;


  (Owner as TLogPanel).SetFocus;

  // run OnMouseMove
  //if assigned(FOnMouseMoved) then
  //   FOnMouseMoved(self);
end;


//=================================================================
// Set new group/background colors
//=================================================================
procedure TLogDisplay.SetColors(Back,Group,Scale,Grid:TColor);
begin
  if    (Back<>BackColor) or (Group<>GroupColor)
     or (Scale<>ScaleColor) or (Grid<>GridColor) then
   begin
     BackColor:=Back;
     GroupColor:=Group;
     ScaleColor:=Scale;
     GridColor:=Grid;
     Redraw(0,0,Width,Height,Bitmap.Canvas);
     Paint;
   end;
end;



//=================================================================
procedure TLogDisplay.Paint;
//=================================================================
begin
  if (Width<>Bitmap.Width) or (Bitmap.Height<>Height) then begin
    Bitmap.Width := Width;
    Bitmap.Height:= Height;
    Redraw(0,0,Width,Height,Bitmap.Canvas);
  end;
  Canvas.CopyRect(Canvas.ClipRect,Bitmap.Canvas,Canvas.ClipRect);
end;


//=================================================================
// Draw waveforms
//=================================================================
procedure TLogDisplay.Redraw(SX,SY,W,H:integer;Canvas:TCanvas);
{$J+} // needed to enable writes to constant
const InPaint : boolean = false;
{$J-}
var
//   smp_n	: real;
   ch		: integer;
   ST,SM,SB	: integer;
   smp_idx	: int64;
   max_smp_idx	: int64;
   act_smp	: cardinal;
   prev_smp	: cardinal;
   prev_x	: integer;
   act_x	: integer;
   d_x		: integer;
   change_found	: boolean;
   chan_val	: string;
   chan_val_size: tsize;
//   oldNotify	:TNotifyEvent;
   Own          : TLogPanel;
begin
   Own:= (Owner as TLogPanel);

   if InPaint then exit;
   InPaint := true;
   try
      // background
      with Canvas do
       begin
	 Pen.Color:=BackColor;
	 Brush.Color:=BackColor;
	 //R:=GetClientRect;	// note: not used because of different area during printing
	 //Rectangle(R.Left,R.Top,R.Right,R.Bottom);
         Rectangle(SX,SY,W,H);
       end;

      if W<11 then exit;                                                // width < 11
      if DataSet.NumberOfLines=0 then exit;                             // no lines to display



      SignalHeight := (H-10) div (DataSet.NumberOfLines+1);
      if SignalHeight > -MaxSignalHeight*Canvas.Font.Height then
	 SignalHeight := round(-MaxSignalHeight*Canvas.Font.Height);


      // draw scale
      ShowScale(SX,SY,W,H,Canvas,false);

      // draw waves
      with Canvas do
       begin
	 ST:=3;							// top (y-coordinate)
	 SB:=ST + SignalHeight*8 div 10;			// bottom (y-coordinate)
	 SM:=(ST+SB) div 2;					// middle (y-coordinate)

	 for ch:=0 to DataSet.NumberOfLines-1 do
	  begin

	    // horizontal grid line
	    if frmSetup.ShowGrid then
	     begin
	       Pen.Color := GridColor;
	       MoveTo(SX+5, SY + ST + 9 * SignalHeight div 10);
	       LineTo(SX+W-10, SY + ST + 9 * SignalHeight div 10);
	     end;

	    if DataSet.Data[ch].Group then
	       Pen.Color := GroupColor
	    else
	       Pen.Color:=DataSet.Data[ch].Color;

	    prev_x := -1;					// 0 = change at x(0), -1 = no change at x(0)
	    smp_idx:=Own.PixelToAddr(Own.Position,0);			// beginning of displayed data
	    max_smp_idx := Own.PixelToAddr(Own.Position,W-10)+1;	// last data index (+1 makes the last sample drawn)

	    // determine whether draw change at x(0)
	    if smp_idx>0 then
	     begin
	       prev_smp := frmMain.DataHold.GetSampleValue(smp_idx-1) and DataSet.Data[ch].Mask;
	       act_smp := frmMain.DataHold.GetSampleValue(smp_idx) and DataSet.Data[ch].Mask;
	       if prev_smp <> act_smp then
		  prev_x := 0;
	       prev_smp := act_smp;
             end
            else
	       prev_smp := frmMain.DataHold.GetSampleValue(smp_idx) and DataSet.Data[ch].Mask;

            // draw
	    repeat
	       change_found := frmMain.DataHold.FindChange(DataSet.Data[ch].Mask, max_smp_idx, true);
	       act_smp := frmMain.DataHold.Sample;

	       act_x := Own.AddrToPixel(Own.Position, frmMain.DataHold.Position);
	       if (act_x<>prev_x) then
		begin
		  if (act_x >= W-10) then
                     act_x := W-10;


		  // group
		  if DataSet.Data[ch].Group then
		   begin
                     d_x := (act_x - prev_x) div 2;
                     if (d_x)>2 then
                        d_x := 2;

                     // = or <
                     if prev_x>=0 then
                      begin
                        MoveTo(SX+prev_x+d_x+5,SY+SB);
                        LineTo(SX+prev_x+0+5,SY+SM);
                        LineTo(SX+prev_x+d_x+5,SY+ST);
                        prev_x := prev_x+d_x;
                      end
                     else
                        prev_x:=0;

                     // = or =>
                     if not(change_found) then
                      begin
			MoveTo(SX+prev_x+5,SY+SB);
                        LineTo(SX+act_x+5,SY+SB);		// _
                        MoveTo(SX+prev_x+5,SY+ST);
                        LineTo(SX+act_x+5,SY+ST);		// -
                      end
                     else
                      begin
                        MoveTo(SX+prev_x+5,SY+SB);
                        LineTo(SX+act_x-d_x+5,SY+SB); 		// _
                        LineTo(SX+act_x+5,SY+SM);		// /
                        LineTo(SX+act_x-d_x+5,SY+ST);		// \
                        LineTo(SX+prev_x+5,SY+ST);		// -
                      end;

		     chan_val := ChannelValueToStr(prev_smp, ch);
		     chan_val_size := TextExtent(chan_val);
		     if     ((chan_val_size.cx+2) < (act_x-prev_x))	// enough space for text?
			and ((chan_val_size.cy) < (SB-ST))then
			TextOut(SX+(2+prev_x)+5,
				SY+((ST+SB) div 2) - (chan_val_size.cy div 2),
				chan_val);
		   end
		  else

		  // single signal
		   begin
                     if prev_x<0 then
                        prev_x:=0;

                     if change_found then
                      begin					// vertical line
                        MoveTo(SX+act_x+5,SY+ST);
                        LineTo(SX+act_x+5,SY+SB);
                      end;

		     if prev_smp = 0 then
		      begin					// draw 0
			MoveTo(SX+prev_x+5,SY+SB);
			LineTo(SX+act_x+5,SY+SB);
			MoveTo(SX+prev_x+5,SY+SB-1);		// draw 0 twice (make it thicker)
			LineTo(SX+act_x+5,SY+SB-1);
		      end
		     else
		      begin					// draw 1
			MoveTo(SX+prev_x+5,SY+ST);
			LineTo(SX+act_x+5,SY+ST);
		      end;
		   end;

		end;
	       prev_x := act_x;
	       prev_smp := act_smp;
	    until not(change_found);
	    inc(ST,SignalHeight);
	    inc(SB,SignalHeight);
	    inc(SM,SignalHeight);

	  end;{ch}
       end;{canvas}

   finally
      InPaint:=false;
   end;
end;

//=================================================================
// Draw scale
//=================================================================
procedure TLogDisplay.ShowScale(SX,SY,W,H:integer;Canvas:TCanvas;Fill:boolean);
var i,ax,tx,ty:integer;
    sdiff,ast,wst:int64;
    precgr : integer;
    s:string;
    R:TRect;
    Own: TLogPanel;
begin
   Own:= (Owner as TLogPanel);

   SignalHeight := (H-10) div (DataSet.NumberOfLines+1);
   if SignalHeight > -MaxSignalHeight*Canvas.Font.Height then
      SignalHeight := round(-MaxSignalHeight*Canvas.Font.Height);

   with Canvas do
    begin
      // draw scale
      if Fill then begin
	Brush.Color:=BackColor;
	R.Top:=H-5-SignalHeight;
	R.Left:=1;
	R.Bottom:=H;
	R.Right:=W;
	FillRect(R);
      end;

      Pen.Color:=ScaleColor;
      Font.Color:=ScaleColor;

      // horizontal line
      MoveTo(SX+5,SY+H-SignalHeight-5);
      LineTo(SX+W-5,SY+H-SignalHeight-5);

      sdiff := round((W-10)*Own.ZoomRatio);                     // number of samples on the screen
      precgr := TmPrec(sdiff / Own.Timebase);

      // print sample numbers
      wst:=GetDecades(sdiff);	   	                        // calculate the step
      ast:= ((Own.Position-Own.Cursors[2].Position) div wst) * wst;  // round to nearest step sample index
      ax := 0;
      while (ax <= W-10) do
       begin
         ax := Own.AddrToPixel(Own.Position, ast+Own.Cursors[2].Position);
	 if (ax >= 0) and (ax <= W-10) then
	  begin
	    // vertical line
            Pen.Color:=ScaleColor;
	    MoveTo(SX+5+ax,SY+H-SignalHeight);
 	    LineTo(SX+5+ax,SY+H-SignalHeight-8);

            // vertical grid line
            if frmSetup.ShowGrid then
             begin
               Pen.Color:=GridColor;
               LineTo(SX+5+ax,SY);
             end;

	    // print sample number/time
            if frmWave.mnScaleSample.Checked then
	       s:=IntToDecStr(ast)
            else
               s:=TmToStrPrec(ast/Own.Timebase, precgr);

	    tx:=5+ax-(TextWidth(s) div 2);
	    if tx<5 then tx:=5;
	    if tx+TextWidth(s)>W-5 then tx:=W-5-TextWidth(s);
            ty := SY+H-SignalHeight+1;
            if ty+TextHeight('0') > H then
               ty := H-TextHeight('0');
	    TextOut(SX+tx,SY+ty,s);
	  end;
	 ast := ast+wst;
       end;

      // draw small vertical lines for individual samples
      Pen.Color:=ScaleColor;
      if Own.ZoomRatio<0.5 then
	for i:=0 to W-10 do
	  if Own.PixelToAddr(Own.Position,i)<>Own.PixelToAddr(Own.Position,i+1) then // next sample?
           begin
	     MoveTo(SX+5+i,SY+H-SignalHeight-8);
	     LineTo(SX+5+i,SY+H-SignalHeight-5);
	   end;
    end;
end;



//=================================================================
// Draw signal names (used for print)
//=================================================================
procedure TLogDisplay.DrawNames(SX,SY,W,H:integer;Canvas:TCanvas);
var
   ch		: integer;
   ST,SM,SB	: integer;
   chtext       : string;
   chsize       : tsize;
//   Own		:TLogPanel;

begin
//  Own:= (Owner as TLogPanel);

   if W<11 then exit;
   if DataSet.NumberOfLines=0 then exit;

   SignalHeight := (H-10) div (DataSet.NumberOfLines+1);
   if SignalHeight > -MaxSignalHeight*Canvas.Font.Height then
      SignalHeight := round(-MaxSignalHeight*Canvas.Font.Height);

   with Canvas do
    begin
      ST:=3;							// top (y-coordinate)
      SB:=ST + SignalHeight*8 div 10;			        // bottom (y-coordinate)
      SM:=(ST+SB) div 2;					// middle (y-coordinate)

      for ch:=0 to DataSet.NumberOfLines-1 do
       begin

         if DataSet.Data[ch].Group then
            Pen.Color := GroupColor
         else
            Pen.Color:=DataSet.Data[ch].Color;

         chtext := IntToStr(ch+1)+'. '+Dataset.Data[ch].Name;
         chsize := TextExtent(chtext);
         TextOut(SX+5,SY+SM - (chsize.cy div 2), chtext);
         inc(SM,SignalHeight);
       end;//ch
    end;//canvas

end;





//==============================================================================
//
//  TLogPanel
//
//==============================================================================



//==============================================================================
//  - create window
//  - create cursors
//==============================================================================
constructor TLogPanel.Create(AOwner:TComponent);
var h: integer;
begin
  inherited Create(AOwner);
  //Height:=320;
  align:=alTop;
  Width:= (AOwner as TPanel).Width;
  h:= (AOwner as TPanel).Height;
  if (h<=0) then h:=100;
  TabStop := true;

  FZoom:=0;
  FMaxZoom := 100;
  FPosition:=int64(0);
  ZoomRatio:=1;

  ActiveCursor := 0;
  ActiveDataset := -1;


  Display       := TLogDisplay.Create(self);
  Display.Parent:= Self;
  Display.Align := alTop;
  Display.Height:= h-23;
  Display.Width:= Width-10;  

  // trigger cursor
  Cursors[2]:=TCursor.Create(self,2);
  with Cursors[2] do begin
    Parent    := self;
    Horizontal:= false;
    Top       := 2;
    Left      := 5;
    Width     := 1;
    Height    := h-25;
    Position  := 0;
    ColorC    := INIFile.ReadInteger('GUI','CURSOR2_COLOR', clRed);
  end;
  frmWave.Scrollbar.SetCursorColor(2, Cursors[2].ColorC);
  frmList.sbrList.SetCursorColor(2, Cursors[2].ColorC);

  // mouse cursor - vertical
  Cursors[3]:=TCursor.Create(self,3);
  with Cursors[3] do begin
    Parent    := self;
    Horizontal:= false;
    Top       := 2;
    Left      := 4;
    Width     := 1;
    Height    := h-25;
    ColorC    := INIFile.ReadInteger('GUI','CURSOR3_COLOR', clAqua);
  end;

  // mouse cursor - horizontal
  Cursors[4]:=TCursor.Create(self,4);
  with Cursors[4] do begin
    Parent    := self;
    Horizontal:= true;
    Top       := h div 2;
    Left      := 2;
    Height    := 1;
    ColorC    := INIFile.ReadInteger('GUI','CURSOR4_COLOR', clAqua);
  end;

  // cursor 1
  Cursors[0]:=TCursor.Create(self,0);
  with Cursors[0] do begin
    Parent    := self;
    Horizontal:= false;
    Top       := 2;
    Left      := 20;
    Width     := 3;
    Height    := h-25;
    ColorC    := INIFile.ReadInteger('GUI','CURSOR0_COLOR', clBlue);
  end;
  frmWave.Scrollbar.SetCursorColor(0, Cursors[0].ColorC);
  frmList.sbrList.SetCursorColor(0, Cursors[0].ColorC);

  // cursor 2
  Cursors[1]:=TCursor.Create(self,1);
  with Cursors[1] do begin
    Parent    := self;
    Horizontal:= false;
    Top       := 2;
    Left      := 20;
    Width     := 3;
    Height    := h-25;
    ColorC    := INIFile.ReadInteger('GUI','CURSOR1_COLOR', clBlue);
  end;
  frmWave.Scrollbar.SetCursorColor(1, Cursors[1].ColorC);
  frmList.sbrList.SetCursorColor(1, Cursors[1].ColorC);


  StatusText:=TLabel.Create(self);
  with StatusText do begin
    Parent     := self;
    Top        := h-15;
    Left       := 2;
    Caption    := 'Position: 0.0%';
    AutoSize   := true;
    Transparent:= true;
  end;

  TimeBase:=100e6;
end;

//=================================================================
destructor TLogPanel.Destroy;
//=================================================================
var i:integer;
begin
  // store colors of cursors
  for i:=0 to 4 do
   begin
     INIFile.WriteInteger('GUI','CURSOR'+IntToStr(i)+'_COLOR', Cursors[i].ColorC);
     Cursors[i].Destroy;
   end;
  inherited Destroy;
end;



//=================================================================
procedure TLogPanel.Paint;
//=================================================================
begin
//   UpdateCursors;
end;




//=================================================================
procedure TLogPanel.SetFocus;
//=================================================================
var
    AWC : TWinControl;
begin
  // set focus to LogPanel (to allow reception of keyboard strokes)
  // note: usage of SetFocus only doesn't work well for MDIChild window
  if not Focused then
   begin
     AWC := frmWave.ActiveControl;
     frmWave.ActiveControl := nil;
     frmWave.SetFocus;
     frmWave.ActiveControl := AWC;
     inherited;
   end;

end;

//=================================================================
procedure TLogPanel.Initialize(Zoom_: real; Pos_: int64; tpos: int64);
//=================================================================
begin
   PrepareDispData(frmWave.LogPanel.Display.DataSet);
   Cursors[2].Position := tpos;
   fSetZoomPos(Zoom_, Pos_);
   //Display.Redraw(0,0,Display.Width,Display.Height,Display.Bitmap.Canvas);
   //Repaint;
   UpdateCursors;
   SetActiveDataset(-1);
end;


//=================================================================
procedure TLogPanel.OnResize;
//=================================================================
var h,w: integer;
begin
  h := Parent.ClientHeight;
  w := Parent.ClientWidth;
  Height := h-20;
  Width := w;

  ZoomRatio:=100*frmMain.DataHold.MaxPosition/(w-10)/FZoom;		           // = samples per pixel for given zoom
  if Round((w-10)*ZoomRatio)+FPosition > frmMain.DataHold.MaxPosition then
    FPosition:=Round(frmMain.DataHold.MaxPosition-(w-10)*ZoomRatio);
  if FPosition<0 then FPosition:=0;

  Display.Height   := Height-23;
//  Cursors[0].Height:= Height-20;
//  Cursors[1].Height:= Height-17;
//  Cursors[3].Height:= Height-27;
//  Cursors[2].Height:= Height-27;
  Cursors[0].Height:= Height-27;
  Cursors[1].Height:= Height-27;
  Cursors[3].Height:= Height-27;
  Cursors[2].Height:= Height-27;
  Cursors[4].Width := Width-4;
  StatusText.Top   := Height-15;
end;



//=================================================================
// Update cursors to new positions
//=================================================================
procedure TLogPanel.UpdateCursors;
var ActCur :integer;
begin
  ActCur := ActiveCursor;
  SetCursorPosition(2,Cursors[2].Position);
//  SetCursorPosition(3,Cursors[3].Position);
  if Cursors[3].Zooming then
     SetCursorPosition(4,Cursors[4].Position);
  if ActCur = 1 then
   begin
     SetCursorPosition(0,Cursors[0].Position);
     SetCursorPosition(1,Cursors[1].Position);
   end
  else
   begin
     SetCursorPosition(1,Cursors[1].Position);
     SetCursorPosition(0,Cursors[0].Position);
   end;
  Cursors[0].Repaint;
  Cursors[1].Repaint;
  Cursors[2].Repaint;
  Cursors[3].Repaint;
  Cursors[4].Repaint;
end;



//=================================================================
// Set Active Dataset
//=================================================================
procedure TLogPanel.SetActiveDataset (No : integer);
begin
   if No<0 then No:=-1;
   if No>=Display.DataSet.NumberOfLines then No:=-1;
   ActiveDataset := No;
   frmWave.sgSigNames.Invalidate;
   frmWave.sgSigValues.Invalidate;
end;

//=================================================================
// Set Active Cursor
//=================================================================
procedure TLogPanel.SetActiveCursor (No : integer);
begin
   if (No<>0) and (No<>1) then exit;
   if No <> ActiveCursor then
    begin
      ActiveCursor := No;
      if Cursors[0].Visible then
         Cursors[0].Invalidate;
      if Cursors[1].Visible then
         Cursors[1].Invalidate;
      frmMain.StatusBar1.Invalidate;            // display new active cursor
    end;

end;


//=================================================================
// Set new zoom (keeps center position)
//=================================================================
procedure TLogPanel.SetZoom(Value: real);
var npos: int64;
begin
   if (Zoom<>MaxZoom) or (Value<MaxZoom) then
    begin
      // calculate position to keep center position unchanged
      npos := round(Position + 50*(frmMain.DataHold.MaxPosition/Zoom - frmMain.DataHold.MaxPosition/Value));
      SetZoomPos(Value, npos);
    end;
end;


//=================================================================
procedure TLogPanel.SetPosition(Value: int64);
//=================================================================
begin
   SetZoomPos(Zoom, Value);
end;


//=================================================================
procedure TLogPanel.SetCenterPosition(Value: int64);
//=================================================================
var npos: int64;
begin
   npos := round(Value - 50*(frmMain.DataHold.MaxPosition/Zoom));
   SetZoomPos(Zoom, npos);
end;


//=================================================================
procedure TLogPanel.SetViewRange(Pos1, Pos2: int64);
//=================================================================
var Z,P:int64;
begin
   Z:=Pos2-Pos1;
   if Z<0 then P:=Pos2
	  else P:=Pos1;
   Z:=Abs(Z);
   if Z=0 then Z:=round(Zoom)
          else Z:=round(90*frmMain.DataHold.MaxPosition / Z);

   if Z>MaxZoom then Z:=round(MaxZoom);

   dec(P,5*frmMain.DataHold.MaxPosition div Z);
   SetZoomPos(Z,P);
end;


//=================================================================
// Set new zoom and data position
//=================================================================
procedure TLogPanel.SetZoomPos(Zoom_: real; Pos_:int64);
begin
   // check ranges
   if      Zoom_ > MaxZoom then Zoom_:= MaxZoom
   else if Zoom_ < 100     then Zoom_:= 100;

   if      Pos_ > frmMain.DataHold.MaxPosition then Pos_:= frmMain.DataHold.MaxPosition
   else if Pos_ < 0            then Pos_:= 0;

   if (Zoom_<>FZoom) or (Pos_<>FPosition) then
    begin
      fSetZoomPos(Zoom_, Pos_);
      Display.Redraw(0,0,Display.Width,Display.Height,Display.Bitmap.Canvas);
      Display.Paint;
      UpdateCursors;
    end;
end;



//=================================================================
// Set new zoom and data position
//=================================================================
procedure TLogPanel.fSetZoomPos(Zoom_: real; Pos_:int64);
var
   smp_n	: real;
   oldNotify	: TNotifyEvent;
   s            : string;
begin
   // check ranges
   if      Zoom_ > MaxZoom then Zoom_:= MaxZoom
   else if Zoom_ < 100     then Zoom_:= 100;

   if      Pos_ > frmMain.DataHold.MaxPosition then Pos_:= frmMain.DataHold.MaxPosition
   else if Pos_ < 0            then Pos_:= 0;

   FZoom := round(Zoom_);
   FPosition := Pos_;

   FMaxZoom := frmMain.DataHold.MaxPosition * 15;         	        // max. zoom - 15% of width per sample

   if FMaxZoom < 100 then
      FMaxZoom := 100;
   if FZoom > FMaxZoom then
      FZoom := FMaxZoom;

   ZoomRatio:=100*frmMain.DataHold.MaxPosition/(Display.Width-10)/FZoom;	// = samples per pixel for given zoom

   if Round((Display.Width-10)*ZoomRatio)+FPosition > frmMain.DataHold.MaxPosition then
     FPosition:=Round(frmMain.DataHold.MaxPosition-(Display.Width-10)*ZoomRatio);
   if FPosition<0 then FPosition:=0;


   // Horizontal scroller
   if (frmWave.Scrollbar<>nil) then
      if FZoom>100 then
       begin
         smp_n:=ZoomRatio*(Display.Width-10);				// number of samples on screen
         frmWave.Scrollbar.Enabled:=true;
         frmWave.Scrollbar.Min:=0;
         frmWave.Scrollbar.PageSize:=0;				        // prevent exceptions when setting max.value
         frmWave.Scrollbar.Max:=frmMain.DataHold.MaxPosition;
         // change position without OnChange event
         oldNotify := frmWave.Scrollbar.OnChange;
         frmWave.Scrollbar.OnChange := nil;
         frmWave.Scrollbar.Position:=FPosition;
         frmWave.Scrollbar.OnChange := oldNotify;
         frmWave.Scrollbar.SmallChange:=Round(0.1*smp_n); 		// 10% of screen width
         frmWave.Scrollbar.LargeChange:=Round(0.8*smp_n); 		// 80% of screen width
         frmWave.Scrollbar.PageSize:=Round(smp_n);			// 100% of screen width
       end
      else
       begin
         frmWave.Scrollbar.Enabled:=false;
         frmWave.Scrollbar.Min:=0;
         frmWave.Scrollbar.Max:=frmMain.DataHold.MaxPosition;
         frmWave.Scrollbar.Position:=0;
         frmWave.Scrollbar.PageSize:=frmWave.Scrollbar.Max;
         FPosition:=0;
       end;

   // Zoom ruler position
   frmMain.tbZoom.Max:=IntLog2(round(FMaxZoom / 100));
   frmMain.tbZoom.LineSize:=(IntLog2(round(FMaxZoom / 100)) div 10);
   frmMain.tbZoom.Frequency:=(IntLog2(round(FMaxZoom / 100)) div 10);
   frmMain.tbZoom.PageSize:=(IntLog2(round(FMaxZoom / 100)) div 10);
   frmMain.tbZoomUpdatePosition(IntLog2(round(FZoom / 100)));

   // update position info
   if frmWave.Scrollbar.Enabled then
    begin
      if frmWave.Scrollbar.Max=frmWave.Scrollbar.PageSize then
         s:='100'
      else
         str(frmWave.Scrollbar.Position/(frmWave.Scrollbar.Max-frmWave.Scrollbar.PageSize)*100:3:1,s);
      s:=s+'%';
    end
   else
      s:='0.0%';
   StatusText.Caption:='Position: '+s;

end;


//=================================================================
// Show cursor number No into the middle of the screen
//=================================================================
procedure TLogPanel.ShowCursor(No: byte);
begin
  if (Cursors[No].Visible = false) then
     SetCenterPosition(Cursors[No].Position)
  else if (No < 2) then
     frmMain.StatusBar1.Invalidate;             // display new active cursor

  // repaint cursors to show new active one
  if No < 2 then
   begin
     ActiveCursor := No;
     Cursors[0].Repaint;
     Cursors[1].Repaint;
   end;
end;

//=================================================================
// Set cursor number No into the middle of the screen
//=================================================================
procedure TLogPanel.SetCursorToHalf(No: byte);
begin
  if No <=2 then
   begin
     SetCursorPosition(No, (Position+PixelToAddr(Position,Display.Width-10)) div 2);
   end;
end;


//=================================================================
// Set position of cursor No to Pos (pos=sample number)
// and snap it to edge in range
//=================================================================
procedure TLogPanel.SetSnapCursorPosition(No: byte; Pos:int64);
var
   i,j : integer;
   mpos,lpos,rpos: int64;
   val: cardinal;
   mask: cardinal;
begin
  if No>1 then
     raise Exception.Create('Snapping works only with cursor 0/1');

  if Pos>frmMain.DataHold.MaxPosition then
    Pos:=frmMain.DataHold.MaxPosition;

  // calculate cursor position
  i:=AddrToPixel(Position, pos);
  if (i>5) and (i<Width-5) and (SelDataset>=0) then
   begin
     mask := Display.Dataset.Data[SelDataset].Mask;

     // left edge
     j:=i-SnapDistance;
     if j<5 then
	j:=5;

     mpos := PixelToAddr(Position,j);
     val := frmMain.DataHold.GetSampleValue(pos) AND Mask;
     lpos := -1;

     if frmMain.DataHold.FindChange(Mask, mpos, false) then
	lpos := frmMain.DataHold.Position+1;

     // right edge
     j:=i+SnapDistance;
     if j>Width-5 then
	j:=Width-5;

     mpos := PixelToAddr(Position,j);
     val := frmMain.DataHold.GetSampleValue(pos) AND Mask;	// reinitialize pointer (start of search)
     rpos := -1;

     if frmMain.DataHold.FindChange(Mask, mpos, true) then
	rpos := frmMain.DataHold.Position;

     // determine distance to nearest edge
     if (lpos=-1) and (rpos=-1) then
	pos := pos				// no change found
     else if (lpos=-1) then
	pos := rpos
     else if (rpos=-1) then
	pos := lpos
     else if (rpos-pos)>(pos-lpos) then
	pos := lpos
     else
	pos := rpos;

   end;

  SetCursorPosition(No, Pos);

  SetActiveDataset(SelDataset);

end;

//=================================================================
// Set position of cursor No to Pos (pos=sample number)
//=================================================================
procedure TLogPanel.SetCursorPosition(No: byte; Pos:int64);
var
   i:int64;
begin
  // calculate visibility
  if (No<3) or ((No=4) and Cursors[No].Zooming) then begin
    if Pos>frmMain.DataHold.MaxPosition then
      Pos:=frmMain.DataHold.MaxPosition;
    // show cursor on the screen? (cursor within display range?)
    Cursors[No].Visible := not ((Pos<Position) or (Pos>PixelToAddr(Position,Width-10)));
  end;

 // calculate position on the screen
 if (Cursors[No].Visible) then
  begin
    // calculate cursor position
    i:=AddrToPixel(Position, pos);
    Cursors[No].Left:=i+5-(Cursors[No].Width) div 2;
  end;

  Cursors[No].Position:=Pos;

  // trigger cursor - necessary to redraw scale
  if No=2 then
   begin
     Display.Redraw(0,0,Display.Width,Display.Height,Display.Bitmap.Canvas);
     Display.Paint;
     //Cursors[0].Invalidate;
     //Cursors[1].Invalidate;
  end;

  // set new active cursor
  if (No < 2) and (No <> ActiveCursor) then
     SetActiveCursor(No)
  else
     Cursors[No].Invalidate;


  // update status bar
  if (No = 2) then
     frmMain.StatusBar1.Panels[1].Text:='Trigger: '+IntToDecStr(Pos)+' ('+TmToStr(Pos/TimeBase,2)+'s)';

  if (No < 3) then
   begin
     frmMain.StatusBar1.Panels[2].Text:='1: '+IntToDecStr(Cursors[0].Position-Cursors[2].Position)+
       ' ('+TmToStr((Cursors[0].Position-Cursors[2].Position)/TimeBase,2)+'s)';
     frmMain.StatusBar1.Panels[3].Text:='2: '+IntToDecStr(Cursors[1].Position-Cursors[2].Position)+
       ' ('+TmToStr((Cursors[1].Position-Cursors[2].Position)/TimeBase,2)+'s)';

     i:=Cursors[1].Position-Cursors[0].Position;
     frmMain.StatusBar1.Panels[4].Text:='Diff.: '+IntToDecStr(i)+' ('+TmToStr(i/TimeBase,2)+'s)';
     frmMain.StatusBar1.Invalidate;
   end;

  // update scrollbar cursors
  if No < 3 then
   begin
     frmWave.Scrollbar.SetCursorPosition(No, pos);
     frmList.SetCursorPosition(No, pos);
   end;

  // update signal info
  if (No < 2) then
   begin
     frmWave.Showdata;
   end;


end;



//=================================================================
// Find next/previous change of the signal and center the screen
//=================================================================
procedure TLogPanel.CenterChange(go_forward: boolean);
var
   pos: int64;
   val: cardinal;
   mask: cardinal;
begin
   if (SelDataset < 0) and (ActiveDataset < 0) then
      exit;

   pos:= Cursors[ActiveCursor].Position;
   if (ActiveDataset >= 0) then
      mask := Display.Dataset.Data[ActiveDataset].Mask
   else
      mask := Display.Dataset.Data[SelDataset].Mask;

   if go_forward then
    begin
      val := (frmMain.DataHold.GetSampleValue(pos) AND Mask);	// seek to sample position
      if frmMain.DataHold.FindChange(Mask, frmMain.DataHold.MaxPosition, true) then
       begin
         SetCursorPosition(ActiveCursor, frmMain.DataHold.Position);
         ShowCursor(ActiveCursor);
       end
    end
   else if (pos > 0) then
    begin
      val := (frmMain.DataHold.GetSampleValue(pos-1) AND Mask);	// seek to sample position
      if frmMain.DataHold.FindChange(Mask, 0, false) then
       begin
         SetCursorPosition(ActiveCursor, frmMain.DataHold.Position+1);
         ShowCursor(ActiveCursor);
       end;
    end;

end;




//=================================================================
// Return sample number (data offset) for given pixel (x-coordinate)
// left align
//=================================================================
function TLogPanel.PixelToAddr(Position:int64; 			// zacatek zobrazovanych dat
                               Point:integer     		// bod na obrazovce vztazeny k 0
                              ):int64;				// navratova hodnota - index do pole
begin
  Result:= Trunc(Point*ZoomRatio)+Position;
  if Result>frmMain.DataHold.MaxPosition then Result:=frmMain.DataHold.MaxPosition;
end;

//=================================================================
// Return sample number (data offset) for given pixel (x-coordinate)
// center align (used for mouse cursor)
//=================================================================
function TLogpanel.CPixelToAddr(Position:int64; 		// zacatek zobrazovanych dat
                                Point:integer     		// bod na obrazovce vztazeny k 0
                               ):int64;        			// navratova hodnota - index do pole
begin
  Result:= Round(Point*ZoomRatio)+Position;
  if Result>frmMain.DataHold.MaxPosition then Result:=frmMain.DataHold.MaxPosition;
end;

//=================================================================
// Return pixel (x-coordinate) for given sample number (data offset)
//=================================================================
function TLogPanel.AddrToPixel(Position:int64; 			// zacatek zobrazovanych dat
                               Address:int64   			// sample number
                              ):integer;        		// navratova hodnota - x coordinate
var px: real;
begin
  px:=trunc((Address-Position)/ZoomRatio);
  if px<low(integer) then
     px := low(integer);
  if px>high(integer) then
     px := high(integer);
  Result:=trunc(px);
end;



//=================================================================
// enable reception of cursor keys keydown events
//=================================================================
procedure TLogPanel.WMGetDlgCode(var Msg: TWMGetDlgCode);
begin
   msg.result:= DLGC_WANTARROWS;
end;

//=================================================================
// Mouse wheel
//=================================================================
procedure TLogPanel.MouseWheel(var Msg: TWMMouseWheel);
var
 Delta: integer;
 IsUp: boolean;
 smp_n : real;
begin
   //Set the wheel value
   Delta := Msg.WheelDelta div WHEEL_DELTA;
   IsUp := Delta > 0;
   if Delta < 0 then Delta := Delta * -1;

   smp_n:=ZoomRatio*(Display.Width-10);

   if IsUp then
    begin
      SetPosition(Position+Round(0.1*smp_n))
    end
   else
    begin
      SetPosition(Position-Round(0.1*smp_n))
    end;

   Msg.Result := 1;
end;


//=================================================================
procedure TLogPanel.KeyDown(var Key: Word; Shift: TShiftState);
//=================================================================
var
   smp_n	: real;
begin
   //frmMain.StatusBar1.Panels[1].Text := 'Pressed: '+IntToStr(key);

   if key in [VK_LEFT, VK_RIGHT, VK_HOME, VK_END, 188, 190] then
    begin
      smp_n:=ZoomRatio*(Display.Width-10);

      case Key of
        VK_LEFT:
          begin
            if (ssAlt in Shift) then
               SetPosition(Position-1)
            else if (ssCtrl in Shift) then
               SetPosition(Position-Round(0.8*smp_n))
            else
               SetPosition(Position-Round(0.1*smp_n));
          end;
        VK_RIGHT:
          begin
            if (ssAlt in Shift) then
               SetPosition(Position+1)
            else if (ssCtrl in Shift) then
               SetPosition(Position+Round(0.8*smp_n))
            else
               SetPosition(Position+Round(0.1*smp_n));
          end;
        VK_HOME:
          SetPosition(0);
        VK_END:
          SetPosition(frmMain.DataHold.MaxPosition-Round(smp_n));
        188:    // '<'
          CenterChange(false);
        190:    // '>'
          CenterChange(true);

      end;
   end;

   Key := 0;
end;


//=================================================================
// Fill TDispData with names from Signal/Bus definitions (frmChannel)
//=================================================================
procedure PrepareDispData(var DispData:TDisplayData);
var i,j,k,channel:integer;
    mask:cardinal;
    s:string;
begin
  channel:=0;

  with frmChannel do
   begin
     FillChar(DispData,SizeOf(DispData),0); // clear target variable

     // signal groups
     for i:=0 to lbGroups.Items.Count-1 do
       with (lbGroups.Items.Objects[i] as TGroupList) do
	 if Count>0 then
	  begin // insert only non-empty group
	    DispData.Data[channel].Base:=Base;
	    DispData.Data[channel].Name:=lbGroups.Items.Strings[i];
	    DispData.Data[channel].Group:=true;
	    DispData.Data[channel].Color:=clSilver;	// later gets predefined (in Redraw)
	    mask:=0;
	    for j:=0 to Count-1 do
	     begin // contantenate channel bits
	       s:=Strings[j];
	       s:=Copy(s,1,Pos('.',s)-1);
	       k:=StrToInt(s)-1;
	       mask:=mask or (1 shl k); // v masce mame bitovou masku channelu
	       DispData.Data[channel].Signals[j]:=k; // signal number
	     end;
	    DispData.Data[channel].Mask:=mask;
	    DispData.Data[channel].NumberOfSignals:=Count;
	    inc(channel);
	  end;

     // single signals
     for i:=0 to MaxChannels-1 do
       with ChannelInfo[i] do
	 if (Group.ItemIndex=0) and Enabled.Checked and Enabled.Enabled then
	  begin
	    DispData.Data[channel].Color:=Edit.Font.Color;
	    DispData.Data[channel].Name:=Edit.Text;
	    DispData.Data[channel].Mask:=1 shl i;
	    DispData.Data[channel].NumberOfSignals:=1;
	    DispData.Data[channel].Signals[0]:=i;
	    inc(channel);
	  end;
     DispData.NumberOfLines:=channel;
   end;
end;




//==============================================================================
//
//  Miscellaneous
//
//==============================================================================




//=================================================================
// Return value of signal/channel in string format
// input - position
//=================================================================
function ChannelPosValueToStr(pos: int64; channel: integer):string;
var
    val  : cardinal;
begin
   val := frmMain.DataHold.GetSampleValue(pos);
   result := ChannelValueToStr(val, channel);
end;


//=================================================================
// Return value of signal/channel in string format
// input - sample
//=================================================================
function ChannelValueToStr(value: cardinal; channel: integer):string;
var
    j    : integer;
    Mask : cardinal;
    res  : cardinal;
begin
   with frmWave.LogPanel.Display do  // Value
    begin
      mask := Dataset.Data[channel].Mask;
      if Dataset.Data[channel].Group then
       begin
	 res:=0;
	 for j:=0 to Dataset.Data[channel].NumberOfSignals-1 do
	   if (value and (1 shl Dataset.Data[channel].Signals[j])) <> 0 then
	      res:=res or (1 shl j);

	 case Dataset.Data[channel].Base of
	   0: result:= IntToBin(res,Dataset.Data[channel].NumberOfSignals, false)+'b';// binary
	   1: result:= IntToOct(res,Dataset.Data[channel].NumberOfSignals)+'o';// oct
	   2: result:= IntToStr(res);// dec
	   3: result:= IntToHex(res,Dataset.Data[channel].NumberOfSignals)+'h';// hex
	 end;
       end
      else if (value and Mask) <> 0 then
	  result:= '1'
      else
	  result:= '0';
      end;
 end;


end.
